// CMain.cpp
// by Comanche

#include "CLog.h"
#include "CButton.h"
#include "CGraphics.h"
#include "CMain.h"
#include "CLandScape.h"
#include "CMenu.h"
#include "CTexture.h"
#include "CTime.h"
#include "CViewport.h"
#include "CColor.h"
#include "CVoxel.h"
#include "CLiquid.h"
#include "CCheckBox.h"

extern CMain GameMain;
extern CLog Log;
extern CLandScape Land;
extern CGraphics Graphics;
extern CMenu Menu;
extern CTime Time;
extern CColor Color;
extern CVoxel Voxel;
extern CButton *Buttons[60];
extern CCheckBox *CheckBoxes[20];
extern CViewport *Viewports[4];
extern CTexture *Textures[1024];
extern CLiquid  *Liquids[128];
extern CTexture *Masks[3];
extern HINSTANCE main_instance;
extern HWND main_window_handle;

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GameInit(HWND main_window_handle)
{		
	Log.WriteToLogFile("> Checking DirectX");
	
	if(!CheckDirectX())
	{			
		Log.WriteErrorToLogFile("> Cannot find DirectX");
		return(FALSE);
	}	

	switch(DXVersion)
	{
		case 0x100:
		Log.WriteToLogFile("> DirectX 1.0 found");
		break;

		case 0x200:
		Log.WriteToLogFile("> DirectX 2.0 found");
		break;

		case 0x300:
		Log.WriteToLogFile("> DirectX 3.0 found");
		break;	

		case 0x500:
		Log.WriteToLogFile("> DirectX 5.0 found");
		break;

		case 0x600:
		Log.WriteToLogFile("> DirectX 6.0 found");
		break;

		case 0x601:
		Log.WriteToLogFile("> DirectX 6.1 found");
		break;

		case 0x700:
		Log.WriteToLogFile("> DirectX 7.0 or better found");
		break;
	}

	if(DXVersion<0x700)
	{
		Log.WriteErrorToLogFile("> This Program cannot run on older Version than DirectX 7.0");
		return(FALSE);
	}

	if(DXPlatform==VER_PLATFORM_WIN32_WINDOWS)
	{
		Log.WriteToLogFile("> Running under Win 9x");
	}

	if(DXPlatform==VER_PLATFORM_WIN32_NT)
	{
		Log.WriteToLogFile("> Running under Win NT");
	}

	CheckProzessor();

	Log.WriteToLogFile("> Checking Prozessor");

	sprintf(buffer,"> Vendor String: %s",VendorString);	
	Log.WriteToLogFile(buffer);

	if(CPUID=TRUE)
	{
		sprintf(buffer,"> Prozessor class: %d86, Model: %d, Stepping ID: %d",
				CpuFamily,CpuModel,CpuStepId);
		Log.WriteToLogFile(buffer);
	}

	if(AMD)
		Log.WriteToLogFile("> Yeah, AMD!");

	if(INTEL)
		Log.WriteToLogFile("> I HATE Intel!");
	
	char features[80];
	ZeroMemory(&features,sizeof(features));
	if (CPUID) sprintf(features,"%s CPUID", features);
	else sprintf(features,"%s Unknown",features);
	if (FPU) sprintf(features,"%s FPU", features);
	if (MMX) sprintf(features,"%s MMX", features);
	if (CMOV) sprintf(features,"%s CMOV", features);
	if (TSC) sprintf(features,"%s TSC",features);
	if (MCA) sprintf(features,"%s MCA",features);
	sprintf(buffer, "> CPU Features:%s", features);
	Log.WriteToLogFile(buffer);

	if (!CPUID)
	{
		Log.WriteErrorToLogFile("> Sorry, no MMX found!");
		return(FALSE);
	}

	if (!MMX) 
	{
		Log.WriteErrorToLogFile("> Sorry, no MMX found!");
		return(FALSE);
	}	
	
	if (DirectDrawCreate(NULL,&lpdd,NULL)!=DD_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create main DirectDraw object");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create main DirectDraw object: OK");

	if (lpdd->QueryInterface(IID_IDirectDraw2,(LPVOID *)&lpdd2)!=S_OK)
	{
		Log.WriteErrorToLogFile("> Error: Query IDirectDraw2 Interface");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create IDirectDraw2 Interface: OK");

	if (lpdd2->SetCooperativeLevel(main_window_handle, 
				DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | 
				DDSCL_ALLOWREBOOT)!=DD_OK)
	{
		Log.WriteErrorToLogFile("> Error: Set exclusive cooperative level ");
		return(FALSE);
	}
	Log.WriteToLogFile("> Set exclusive cooperative level: OK");

	if (lpdd2->SetDisplayMode(SCREEN_WIDTH,SCREEN_HEIGHT,SCREEN_BPP,0,NULL)!=DD_OK)
	{
		sprintf(buffer,"> Error: Set %dx%d display mode",SCREEN_WIDTH,SCREEN_HEIGHT);
		Log.WriteErrorToLogFile(buffer);
		return(FALSE);
	}
	sprintf(buffer,"> Set %dx%d display mode: OK",SCREEN_WIDTH,SCREEN_HEIGHT);
	Log.WriteToLogFile(buffer);

	memset(&ddsd,0,sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP |
		                  DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;

	if(lpdd2->CreateSurface(&ddsd,&lpddsprimary,NULL)!=DD_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create primary surface");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create primary surface: OK");
	
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
	if(lpddsprimary->GetAttachedSurface(&ddscaps,&lpddsback)!=DD_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create back buffer");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create back buffer: OK");

	memset(&ddsd,0,sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH; 
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
	ddsd.dwWidth = SCREEN_WIDTH;
	ddsd.dwHeight = SCREEN_HEIGHT;
	if(lpdd2->CreateSurface(&ddsd,&lpddsos1,NULL)!=DD_OK)
	{
		sprintf(buffer,"> Error: Create %dx%d off-screen surface",
				SCREEN_WIDTH,SCREEN_HEIGHT);
		Log.WriteErrorToLogFile(buffer);
		return(FALSE);
	}
	sprintf(buffer,"> Create %dx%d off-screen surface: OK",SCREEN_WIDTH,SCREEN_HEIGHT);
	Log.WriteToLogFile(buffer);

	int alloc=(SCREEN_HEIGHT*SCREEN_WIDTH*(SCREEN_BPP/8)*3)/1024;
	sprintf(buffer,"> Allocated %d KByte of video memory",alloc);
	Log.WriteToLogFile(buffer);

	int memory=(2000*2000*4)/1024;
	sprintf(buffer,"> Allocated %d KByte for map data",memory);
	Log.WriteToLogFile(buffer);	

	return(TRUE);
} // end GameInit

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::CheckProzessor()
{
	int OpCPUID=0;
	int OpMMX=0;
	int OpFPU=0;
	int OpCMOV=0;
	int OpMCA=0;
	int OpTSC=0;	
	
	char buffer2[32];
	
	int CFT,CMT,CSIT;
	
	ZeroMemory(&buffer2,sizeof(buffer2));

	__asm
	{	
		pushad			
		pushfd
		pushfd
		pop ebx
		xor ebx,0100000h					// test for CPUID
		push ebx							// Flag 21 des EFlag Registers 
		popfd								// ist setzbar wenn CPUID untersttzt
		pushfd								// wird.
		pop ebx
		test ebx,0100000h
		jz jmp1		
		popfd
		mov OpCPUID,0
		jmp end
jmp1:	popfd
		mov OpCPUID,1

		mov eax,0							// CPUID 
		cpuid
		cmp eax,0							// Nochmals berprfen
		jz end
		
		mov DWORD PTR [buffer2],ebx			// Vendor ID in ebx,edx und ecx
		mov DWORD PTR [buffer2+4],edx
		mov DWORD PTR [buffer2+8],ecx

		mov eax,1							// CPUID
		cpuid	
		
		mov ebx,eax							// Prozessorklasse in EAX
		and ebx,0780h						// Bits 8-11
		shr ebx,8
		mov CFT,ebx
		
		mov ebx,eax							// Model in EAX Bits 4-8
		and ebx,0F0h
		shr ebx,4
		mov CMT,ebx

		mov ebx,eax							// Stepping ID in EAX Bits 0-3
		and ebx,0Fh
		mov CSIT,ebx

		mov eax,1
		cpuid

		test edx,08h						// test for TSC (Bit 4 in edx nach CPUID)
		jz notsc
		mov OpTSC,1
		jmp tsc
notsc:	mov OpTSC,0

tsc:	test edx,04000h						// test for MCA (Bit 14 in edx nach CPUID)
		jz nomca
		mov OpMCA,1
		jmp mca
nomca:	mov OpMCA,0

mca:	test edx,0800000h					// test for MMX (Bit 23 in edx nach CPUID)
		jz nommx					
		mov OpMMX,1
		jmp mmx
nommx:	mov OpMMX,0

mmx:	test edx,010000h					// test for CMOV (Bit 15 in edx nach CPUID)
		jz nocmov
		mov OpCMOV,1
		jmp cmov
nocmov:	mov OpCMOV,0

cmov:	test edx,01h						// test for FPU (Bit 0 in edx nach CPUID)
		jz nofpu
		mov OpFPU,1
		jmp end
nofpu:	mov OpFPU,0

end:	popad
	}

	if(OpCPUID) 
	{
		CPUID=TRUE; 
		strcpy(VendorString,buffer2);
	}
	else
	{
		CPUID=FALSE;
		strcpy(VendorString,"Unknown");
	}	

	AMD=FALSE;
	INTEL=FALSE;

	for(int i=0;i<29;i++)
	{
		if(strcmp(&VendorString[i],"AMD")==0)
		{
			AMD=TRUE;
		}
	}

	for(i=0;i<27;i++)
	{
		if(strcmp(&VendorString[i],"Intel")==0)
		{
			INTEL=TRUE;
		}
	}

	if(OpFPU) FPU=TRUE; else FPU=FALSE;
	if(OpMMX) MMX=TRUE; else MMX=FALSE;
	if(OpTSC) TSC=TRUE; else TSC=FALSE;
	if(OpCMOV) CMOV=TRUE; else CMOV=FALSE;
	if(OpMCA) MCA=TRUE; else MCA=FALSE;

	CpuFamily = CFT;
	CpuModel = CMT;
	CpuStepId = CSIT;

	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::CheckDirectX()
{
    HRESULT              hr;
    HINSTANCE            DDHinst = 0;
    HINSTANCE            DIHinst = 0;
    LPDIRECTDRAW         lpDDraw  = 0;
    LPDIRECTDRAW2        lpDDraw2 = 0;
    LPDIRECTDRAW7		 lpDDraw7 = 0;
	DIRECTDRAWCREATE     DirectDrawCreate   = 0;
    DIRECTDRAWCREATEEX   DirectDrawCreateEx = 0;
    DIRECTINPUTCREATE    DirectInputCreate  = 0;
    OSVERSIONINFO        OSVersion;
    LPDIRECTDRAWSURFACE  lpSurf  = 0;
    LPDIRECTDRAWSURFACE3 lpSurf3 = 0;
    LPDIRECTDRAWSURFACE4 lpSurf4 = 0;

    // First get the windows platform
    OSVersion.dwOSVersionInfoSize = sizeof(OSVersion);    
	if(!GetVersionEx(&OSVersion))
    {
        DXPlatform = 0;
        DXVersion  = 0;
        return(FALSE);
    }

    if( OSVersion.dwPlatformId == VER_PLATFORM_WIN32_NT )
    {
        DXPlatform = VER_PLATFORM_WIN32_NT;

        // NT is easy... NT 4.0 is DX2, 4.0 SP3 is DX3, 5.0 is DX5
        // and no DX on earlier versions.
        if( OSVersion.dwMajorVersion < 4 )
        {
            DXVersion = 0; // No DX on NT3.51 or earlier
            return(FALSE);
        }

        if( OSVersion.dwMajorVersion == 4 )
        {
            // NT4 up to SP2 is DX2, and SP3 onwards is DX3, so we are at least DX2
            DXVersion = 0x200;

            // We're not supposed to be able to tell which SP we're on, so check for dinput
            DIHinst = LoadLibrary( "DINPUT.DLL" );
            if( DIHinst == 0 )
            {
                // No DInput... must be DX2 on NT 4 pre-SP3
                OutputDebugString( "Couldn't LoadLibrary DInput\r\n" );
                return(TRUE);
            }

            DirectInputCreate = (DIRECTINPUTCREATE)GetProcAddress( DIHinst,
                                                                 "DirectInputCreateA" );
            FreeLibrary( DIHinst );

            if( DirectInputCreate == 0 )
            {
                // No DInput... must be pre-SP3 DX2
                OutputDebugString( "Couldn't GetProcAddress DInputCreate\r\n" );
                return(TRUE);
            }

            // It must be NT4, DX2
            DXVersion = 0x300;  // DX3 on NT4 SP3 or higher
            return(TRUE);
        }
        // Else it's NT5 or higher, and it's DX5a or higher: Drop through to
        // Win9x tests for a test of DDraw (DX6 or higher)
    }
    else
    {
        // Not NT... must be Win9x
        DXPlatform = VER_PLATFORM_WIN32_WINDOWS;
    }

    // Now we know we are in Windows 9x (or maybe 3.1), so anything's possible.
    // First see if DDRAW.DLL even exists.
    DDHinst = LoadLibrary( "DDRAW.DLL" );
    if( DDHinst == 0 )
    {
        DXVersion  = 0;
        DXPlatform = 0;
        FreeLibrary( DDHinst );
        return(FALSE);
    }

    // See if we can create the DirectDraw object.
    DirectDrawCreate = (DIRECTDRAWCREATE)GetProcAddress( DDHinst, "DirectDrawCreate" );
    if( DirectDrawCreate == 0 )
    {
        DXVersion  = 0;
        DXPlatform = 0;
        FreeLibrary( DDHinst );
        OutputDebugString( "Couldn't LoadLibrary DDraw\r\n" );
        return(FALSE);
    }

    hr = DirectDrawCreate( NULL, &lpDDraw, NULL );
    if( FAILED(hr) )
    {
        DXVersion  = 0;
        DXPlatform = 0;
        FreeLibrary( DDHinst );
        OutputDebugString( "Couldn't create DDraw\r\n" );
        return(FALSE);
    }

    // So DirectDraw exists.  We are at least DX1.
    DXVersion = 0x100;

    // Let's see if IID_IDirectDraw2 exists.
    hr = lpDDraw->QueryInterface( IID_IDirectDraw2, (VOID**)&lpDDraw2 );
    if( FAILED(hr) )
    {
        // No IDirectDraw2 exists... must be DX1
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        OutputDebugString( "Couldn't QI DDraw2\r\n" );
        return(TRUE);
    }

    // IDirectDraw2 exists. We must be at least DX2
    lpDDraw2->Release();
    DXVersion = 0x200;


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 3.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // DirectInput was added for DX3
    DIHinst = LoadLibrary( "DINPUT.DLL" );
    if( DIHinst == 0 )
    {
        // No DInput... must not be DX3
        OutputDebugString( "Couldn't LoadLibrary DInput\r\n" );
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    DirectInputCreate = (DIRECTINPUTCREATE)GetProcAddress( DIHinst,
                                                        "DirectInputCreateA" );
    if( DirectInputCreate == 0 )
    {
        // No DInput... must be DX2
        FreeLibrary( DIHinst );
        FreeLibrary( DDHinst );
        lpDDraw->Release();
        OutputDebugString( "Couldn't GetProcAddress DInputCreate\r\n" );
        return(TRUE);
    }

    // DirectInputCreate exists. We are at least DX3
    DXVersion = 0x300;
    FreeLibrary( DIHinst );

    // Can do checks for 3a vs 3b here


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 5.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // We can tell if DX5 is present by checking for the existence of
    // IDirectDrawSurface3. First, we need a surface to QI off of.
    DDSURFACEDESC ddsd;
    ZeroMemory( &ddsd, sizeof(ddsd) );
    ddsd.dwSize         = sizeof(ddsd);
    ddsd.dwFlags        = DDSD_CAPS;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    hr = lpDDraw->SetCooperativeLevel( NULL, DDSCL_NORMAL );
    if( FAILED(hr) )
    {
        // Failure. This means DDraw isn't properly installed.
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        DXVersion = 0;
        OutputDebugString( "Couldn't Set coop level\r\n" );
        return(FALSE);
    }

    hr = lpDDraw->CreateSurface( &ddsd, &lpSurf, NULL );
    if( FAILED(hr) )
    {
        // Failure. This means DDraw isn't properly installed.
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        DXVersion = 0;
        OutputDebugString( "Couldn't CreateSurface\r\n" );
        return(FALSE);
    }

    // Query for the IDirectDrawSurface3 interface
    if( FAILED( lpSurf->QueryInterface( IID_IDirectDrawSurface3,
                                       (VOID**)&lpSurf3 ) ) )
    {
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    // QI for IDirectDrawSurface3 succeeded. We must be at least DX5
    DXVersion = 0x500;


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 6.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // The IDirectDrawSurface4 interface was introduced with DX 6.0
    if( FAILED( lpSurf->QueryInterface( IID_IDirectDrawSurface4,
                                       (VOID**)&lpSurf4 ) ) )
    {
        lpDDraw->Release();
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    // IDirectDrawSurface4 was create successfully. We must be at least DX6
    DXVersion = 0x600;
    lpSurf->Release();
    lpDDraw->Release();


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 6.1 Checks
    ///////////////////////////////////////////////////////////////////////////

    // Check for DMusic, which was introduced with DX6.1
    LPDIRECTMUSIC lpDMusic = NULL;
    CoInitialize( NULL );
    hr = CoCreateInstance( CLSID_DirectMusic, NULL, CLSCTX_INPROC_SERVER,
                           IID_IDirectMusic, (VOID**)&lpDMusic );
    if( FAILED(hr) )
    {
        OutputDebugString( "Couldn't create CLSID_DirectMusic\r\n" );
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    // DirectMusic was created successfully. We must be at least DX6.1
    DXVersion = 0x601;
    lpDMusic->Release();
    CoUninitialize();
    

    ///////////////////////////////////////////////////////////////////////////
    // DirectX 7.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // Check for DirectX 7 by creating a DDraw7 object
    LPDIRECTDRAW7 pDD7;
    DirectDrawCreateEx = (DIRECTDRAWCREATEEX)GetProcAddress( DDHinst,
                                                       "DirectDrawCreateEx" );
    if( NULL == DirectDrawCreateEx )
    {
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    if( FAILED( DirectDrawCreateEx( NULL, (VOID**)&lpDDraw7, IID_IDirectDraw7,
                                    NULL ) ) )
    {
        FreeLibrary( DDHinst );
        return(TRUE);
    }

    // DDraw7 was created successfully. We must be at least DX7.0
    DXVersion = 0x700;
    lpDDraw7->Release();

    
    ///////////////////////////////////////////////////////////////////////////
    // End of checks
    ///////////////////////////////////////////////////////////////////////////

    // Close open libraries and return
    FreeLibrary( DDHinst );
    
    return(TRUE);	
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GameStart()
{
	DrawCursor=TRUE;
	
	Graphics.DrawFont = TRUE;
	Graphics.logfont.lfHeight = STANDARTFONTHEIGHT;
	Graphics.logfont.lfWidth = 0;
	Graphics.logfont.lfEscapement = 0;
	Graphics.logfont.lfOrientation = 0;
	Graphics.logfont.lfWeight = FW_BOLD;
	Graphics.logfont.lfItalic = FALSE;
	Graphics.logfont.lfUnderline = FALSE;
	Graphics.logfont.lfStrikeOut = FALSE;
	Graphics.logfont.lfCharSet = OEM_CHARSET;
	Graphics.logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	Graphics.logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	Graphics.logfont.lfQuality = PROOF_QUALITY;
	Graphics.logfont.lfPitchAndFamily = DEFAULT_PITCH;
	lstrcpy(Graphics.logfont.lfFaceName, "Arial"); 

	Viewports[0] = new CViewport(135,20,VIEWPORTTOP);
	Viewports[1] = new CViewport(405,20,VIEWPORTFRONT);
	Viewports[2] = new CViewport(135,260,VIEWPORTLEFT);
	Viewports[3] = new CViewport(405,260,VIEWPORT3D);

	Graphics.BlackQuader(0,0,640,480);

	Graphics.Lock();

	// Land Editor Buttons

	Buttons[0] = new CButton(10,440,90,460,"Ende","");
	Buttons[1] = new CButton(1,20,100,40,"Menu >","");
	Buttons[2] = new CButton(562,8,570,239,"","");
	Buttons[3] = new CButton(562,240,570,462,"","");
	Buttons[4] = new CButton(2,60,32,90,"","data\\Button1.bmp");		//klein
	Buttons[5] = new CButton(35,60,65,90,"","data\\Button2.bmp");		//mittel
	Buttons[6] = new CButton(68,60,98,90,"","data\\Button3.bmp");		//gro
	Buttons[7] = new CButton(2,92,32,122,"","data\\Button4.bmp");		//higher
	Buttons[8] = new CButton(35,92,65,122,"","data\\Button5.bmp");		//lower
	Buttons[9] = new CButton(68,92,98,122,"","data\\Button6.bmp");		//brush
	Buttons[10] = new CButton(2,124,32,154,"","data\\Button7.bmp");		//smoth
	Buttons[11] = new CButton(35,124,65,154,"","data\\Button8.bmp");	//level
	Buttons[12] = new CButton(68,124,98,154,"","data\\Button9.bmp");	//redraw
	Buttons[13] = new CButton(2,156,32,186,"","data\\Button10.bmp");    //spray
	Buttons[14] = new CButton(35,156,65,186,"","data\\Button11.bmp");   //transparent
	Buttons[15] = new CButton(68,156,98,186,"","data\\Button12.bmp");	//liquid mode

	// Voxel Editor Buttons

	Buttons[30] = new CButton(2,50,32,70,"","data\\ArrowUp.bmp");
	Buttons[31] = new CButton(35,50,65,70,"","data\\ArrowUp.bmp");
	Buttons[32] = new CButton(68,50,98,70,"","data\\ArrowUp.bmp");
	Buttons[33] = new CButton(2,144,32,164,"","data\\ArrowDown.bmp");
	Buttons[34] = new CButton(35,144,65,164,"","data\\ArrowDown.bmp");
	Buttons[35] = new CButton(68,144,98,164,"","data\\ArrowDown.bmp");

	// Check Boxes

	CheckBoxes[0] = new CCheckBox(6,355,"Lock Screen");	
	CheckBoxes[1] = new CCheckBox(6,370,"Draw VPF");
	CheckBoxes[2] = new CCheckBox(6,385,"ActiveLiquid");
	CheckBoxes[3] = new CCheckBox(6,400,"Outlines Off");
	CheckBoxes[4] = new CCheckBox(6,415,"Only Outlines");

	Graphics.DrawButton(0,0,100,479);
	Graphics.DrawButton(560,0,639,479);
	Graphics.DrawPushedButton(13,267,84,338);		// MiniMap
	Graphics.DrawPushedButton(4,215,96,255);
	for(int i=0;i<96;i++)
		Graphics.HLine(2+i,2,15,0,0,240-i);
	Graphics.UnLock();

	Menu.Selection=-1;
	Menu.DrawTextures();

	Menu.DrawMiniMap();
	
	Graphics.logfont.lfHeight = SMALFONTHEIGHT;
	Graphics.OSGetDC();
	Graphics.Text(50,5,Version);
	Graphics.OSReleaseDC();	
	Graphics.logfont.lfHeight = STANDARTFONTHEIGHT;

	Graphics.BltToBackBuffer();

	Graphics.Flip();

	Cursor = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_CURSOR));
	Cross = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_CROSS));
	Finger = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_STOP));
	Pinpet = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_PINPET)); 
	Clock = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_CLOCK));
	Right = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_RIGHT)); 
	Left = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_LEFT)); 
	Up = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_UP)); 
	Down = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_DOWN)); 
	RightDown = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_DOWNRIGHT)); 
	RightUp = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_UPRIGHT)); 
	LeftDown = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_DOWNLEFT)); 
	LeftUp = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_UPLEFT)); 
	LeftRight = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_LEFTRIGHT));
	UpDown = LoadIcon(main_instance,MAKEINTRESOURCE(IDI_UPDOWN));
	Log.WriteToLogFile("> Loading Cursors: OK");	

	Land.Day=TRUE;
	ActiveLiquid=-1;

	return(TRUE);
} // end GameStart

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GameMain()
{
	if(Editor==0)
	{
		GetMouseInfo();
		GetKeyboardInfo();

		Land.vps=0;

		if(CheckMouseRange(3,2,99,17,TRUE))
			Menu.MessageBox("Credits","VoxEd by Comanche",MS_OK);		
		
		if (Buttons[0]->TestIfPressed()) 
		{	
			if(Menu.MessageBox("VoxEd","Do you realy want to exit?",MS_YESNO))
			{
				PostMessage(main_window_handle,WM_DESTROY,0,0);
				return(TRUE);
			}
		}

		if (Buttons[1]->TestIfPressed())
		{
			Menu.Menu();
			return(TRUE);
		}
		if (Buttons[2]->TestIfPressed()) Menu.ScrollUp();
		if (Buttons[3]->TestIfPressed()) Menu.ScrollDown();
		if (Buttons[4]->TestIfPressed()) Status=0;
		if (Buttons[5]->TestIfPressed()) Status=1;
		if (Buttons[6]->TestIfPressed()) Status=2;
		if (Buttons[7]->TestIfPressed()) Status2=0;
		if (Buttons[8]->TestIfPressed()) Status2=1;
		if (Buttons[9]->TestIfPressed()) Status2=2;
		if (Buttons[10]->TestIfPressed()) Status2=3;
		if (Buttons[11]->TestIfPressed()) Status2=4;
		if (Buttons[12]->TestIfPressed()) 
		{
			Graphics.Lock();
			Land.InitScreen();
			Graphics.UnLock();
		}
		if (Buttons[13]->TestIfPressed()) Status2=5;
		if (Buttons[14]->TestIfPressed()) Status2=6;
		if (Buttons[15]->TestIfPressed()) Status2=7;

		CheckBoxes[0]->TestIfPressed();
		CheckBoxes[1]->TestIfPressed();
		CheckBoxes[2]->TestIfPressed();
		CheckBoxes[3]->TestIfPressed();
		CheckBoxes[4]->TestIfPressed();
		
		if (KeyboardB[0]=='R')
		{			
			Graphics.Lock();
			Land.InitScreen();
			Graphics.UnLock();
		}
		if (KeyboardB[0]==219)
		{
			Buttons[1]->RedrawPushedButton();
			Menu.Menu();
			Buttons[1]->RedrawButton();
			return(TRUE);
		}
		if (KeyboardB[0]==56 && KeyboardB[1]==62)
		{
			PostMessage(main_window_handle,WM_DESTROY,0,0);
		}
		
		if (CheckMouseRange(572,8,636,463,TRUE))
			Menu.SelectTexture();
		
		if (MouseX>100 && MouseX<560 && MouseB[0])
			Land.UseBrush();	

		if (MouseX>100 && MouseX<560 && MouseB[1])
		{	
			Land.UseBrush();
			while(MouseB[1])
			{
				GetMouseInfo();
			}
		}		
	
		Icon=Land.CheckMouseCursor();
	
		Graphics.BltToBackBuffer();		

		if(Status2==7)
		{
			if(KeyboardB[0]==211 && ActiveLiquid!=-1)
			{
				delete Liquids[ActiveLiquid];
				Liquids[ActiveLiquid]=NULL;
			}

			if(KeyboardB[0]=='-' && ActiveLiquid!=-1)
			{
				if(Liquids[ActiveLiquid]->height>0)
					Liquids[ActiveLiquid]->height-=1;
			}

			if(KeyboardB[0]=='+' && ActiveLiquid!=-1)
			{
				if(Liquids[ActiveLiquid]->height<32)
					Liquids[ActiveLiquid]->height+=1;
			}

			RenderLiquids();
			CheckLiquids();
			if(MouseX>100 && MouseX<560 && MouseB[0] 
				&& Menu.Selection>-1 && ActiveLiquid==-1)
			{
				for(int i=0;i<128;i++)
				{
					if(!Liquids[i])
					{
						Liquids[i] = new CLiquid(Menu.Selection);
						i=128;
					}
				}
			}		
		}

		Land.DrawBrush();

		Graphics.GetDC();
		if (MouseX>100 && MouseX<560 && Status2==7)
			DrawIcon(Graphics.xdc,MouseX-7,MouseY-7,Icon);
		if (MouseX<101 || MouseX>559)
			DrawIcon(Graphics.xdc,MouseX+CursorRegX,MouseY+CursorRegY,Cursor);
		Graphics.ReleaseDC();		

		Graphics.Flip();
		return(TRUE);
	}

	if(Editor==1)
	{
		GetMouseInfo();
		GetKeyboardInfo();

		if(CheckMouseRange(3,2,99,17,TRUE))
			Menu.MessageBox("Credits","VoxEd by Comanche",MS_OK);		

		if (Buttons[0]->TestIfPressed()) 
		{
			PostMessage(main_window_handle,WM_DESTROY,0,0);
			return(TRUE);
		}
		if (Buttons[1]->TestIfPressed()) 
		{
			Menu.Menu();
			return(TRUE);
		}
		
		if(Buttons[30]->TestIfPressed())
		{
			if(Color.redpos<10)
				Color.redpos=0;
			else Color.redpos-=10;
			Color.DrawColors();
		}
		if(Buttons[31]->TestIfPressed())
		{
			if(Color.greenpos<10)
				Color.greenpos=0;
			else Color.greenpos-=10;
			Color.DrawColors();
		}
		if(Buttons[32]->TestIfPressed())
		{
			if(Color.bluepos<10)
				Color.bluepos=0;
			else Color.bluepos-=10;
			Color.DrawColors();
		}
		if(Buttons[33]->TestIfPressed())
		{
			if(Color.redpos>239)
				Color.redpos=249;
			else Color.redpos+=10;
			Color.DrawColors();
		}
		if(Buttons[34]->TestIfPressed())
		{
			if(Color.greenpos>239)
				Color.greenpos=249;
			else Color.greenpos+=10;
			Color.DrawColors();
		}			
		if(Buttons[35]->TestIfPressed())
		{
			if(Color.bluepos>239)
				Color.bluepos=249;
			else Color.bluepos+=10;				
			Color.DrawColors();
		}

		Color.TestIfPressed();

		if (KeyboardB[0]==219)
		{
			Buttons[1]->RedrawPushedButton();
			Menu.Menu();
			Buttons[1]->RedrawButton();
			return(TRUE);
		}
		if (KeyboardB[0]==56 && KeyboardB[1]==62)
		{
			PostMessage(main_window_handle,WM_DESTROY,0,0);
		}
		
		Graphics.BltToBackBuffer();

		Voxel.ViewX=0; Voxel.ViewY=0;
		Viewports[0]->UseViewport();
		Viewports[1]->UseViewport();
		Viewports[2]->UseViewport();
		Viewports[3]->UseViewport();

		HICON icon=Cursor;

		Graphics.GetDC();
		DrawIcon(Graphics.xdc,MouseX+CursorRegX,MouseY+CursorRegY,icon);
		Graphics.ReleaseDC();

		Graphics.Flip();
		return(TRUE);
	}

	return(TRUE);
} // end GameMain

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GameShutdown()
{
	int i;
	int i2=0;
	Log.WriteStatusToLogFile("> GameShutdown: Shutting down game");

	i2=0;
	for(i=0;i<128;i++)
	{		
		if(Liquids[i])
		{
			i2++;
			delete Liquids[i];
		}
	}
	sprintf(buffer,"> %d Liquids deleted",i2);
	Log.WriteToLogFile(buffer);

	i2=0;
	for(i=0;i<60;i++)
	{
		if(Buttons[i])
		{
			i2++;
			delete Buttons[i];
		}
	}
	sprintf(buffer,"> %d Buttons deleted",i2);
	Log.WriteToLogFile(buffer);

	i2=0;
	for(i=0;i<20;i++)
	{
		if(CheckBoxes[i])
		{
			i2++;
			delete CheckBoxes[i];
		}
	}
	sprintf(buffer,"> %d Check Boxes deleted",i2);
	Log.WriteToLogFile(buffer);

	i2=0;
	for(i=0;i<1024;i++)
	{
		if(Textures[i])
		{
			i2++;
			delete Textures[i];
		}
	}
	sprintf(buffer,"> %d Textures deleted",i2);
	Log.WriteToLogFile(buffer);

	if(Masks[0]) delete Masks[0];
	if(Masks[1]) delete Masks[1];
	if(Masks[2]) delete Masks[2];
	Log.WriteToLogFile("> 3 Masks deleted");

	if(Viewports[0]) delete Viewports[0];
	if(Viewports[1]) delete Viewports[1];
	if(Viewports[2]) delete Viewports[2];
	if(Viewports[3]) delete Viewports[3];
	Log.WriteToLogFile("> 4 Viewports deleted");

	if (lpddsprimary!=NULL)
		lpddsprimary->Release();
	Log.WriteToLogFile("> Release primary surface: OK");
	
	if(lpddsos1!=NULL)
		lpddsos1->Release();
	Log.WriteToLogFile("> Release off-screen surface: OK");

	if (lpdd!=NULL)
		lpdd->Release();
	Log.WriteToLogFile("> Release DirectDraw object: OK");

	if (lpdidevice!=NULL)
	{ 
		lpdidevice->Unacquire();      
		lpdidevice->Release();
		lpdidevice=NULL;
	}
	Log.WriteToLogFile("> Unacquire mouse: OK");
	
	if(lpdidevice2!=NULL)
	{
		lpdidevice2->Unacquire();
		lpdidevice2->Release();
		lpdidevice2=NULL;
	}
	Log.WriteToLogFile("> Unacquire keyboard: OK");

	if (lpdi!=NULL)   
	{
		lpdi->Release();
		lpdi=NULL;
	}
	Log.WriteToLogFile("> Release DirectInput object: OK");

	Log.WriteToLogFile("> GameShutdown: OK");
	Log.CloseLogFile();

	return(TRUE);
} // end GameShutdown

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GameRestart()
{
	Graphics.RestorePrimarySurface();
	Graphics.RestoreOffScreenSurface();
	Graphics.BlackQuader(0,0,640,480);

	if(Editor==0)
	{
		Graphics.Lock();
		Graphics.DrawButton(0,0,100,479);
		Graphics.DrawButton(560,0,639,479);
		Graphics.DrawPushedButton(13,267,84,338);		// MiniMap
		Graphics.DrawPushedButton(4,215,96,255);
		for(int i=0;i<96;i++)
			Graphics.HLine(2+i,2,15,0,0,240-i);
		Land.InitScreen();
		Graphics.UnLock();		
		Menu.DrawTextures();
		Menu.DrawMiniMap();
		Log.WriteToLogFile("> Redraw Graphics: OK");
		for(i=0;i<30;i++)
		{
			if(Buttons[i])
				Buttons[i]->RedrawButton();
		}
		for(i=0;i<20;i++)
		{
			if(CheckBoxes[i])
				CheckBoxes[i]->RedrawCheckBox();
		}
		Graphics.logfont.lfHeight = SMALFONTHEIGHT;
		Graphics.OSGetDC();
		Graphics.Text(50,5,Version);
		Graphics.OSReleaseDC();	
		Graphics.logfont.lfHeight = STANDARTFONTHEIGHT;
		Graphics.BltToBackBuffer();
		Graphics.Flip();
		return(TRUE);
	}

	if(Editor==1)
	{
		Graphics.Lock();
		Graphics.DrawButton(0,0,100,479);
		Graphics.DrawPushedButton(4,215,96,255);
		Graphics.DrawButton(101,0,369,239);
		Graphics.DrawButton(370,0,639,239);
		Graphics.DrawButton(101,240,369,479);
		Graphics.DrawButton(370,240,639,479);		
		for(int i=0;i<96;i++)
			Graphics.HLine(2+i,2,15,0,0,200-i);
		Graphics.UnLock();
		Viewports[0]->RedrawViewport();
		Viewports[1]->RedrawViewport();
		Viewports[2]->RedrawViewport();
		Viewports[3]->RedrawViewport();
		Log.WriteToLogFile("> Redraw Graphics: OK");
		Buttons[0]->RedrawButton();
		Buttons[1]->RedrawButton();
		for(i=30;i<60;i++)
		{
			if(Buttons[i])
				Buttons[i]->RedrawButton();
		}
		Graphics.logfont.lfHeight = SMALFONTHEIGHT;
		Graphics.OSGetDC();
		Graphics.Text(50,5,Version);
		Graphics.OSReleaseDC();	
		Graphics.logfont.lfHeight = STANDARTFONTHEIGHT;
		Color.DrawColors();
		Graphics.BltToBackBuffer();
		Graphics.Flip();
		return(TRUE);
	}

	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::MouseInit(HWND hwnd)
{
	if (DirectInputCreate(main_instance,DIRECTINPUT_VERSION,&lpdi,NULL)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create main DirectInput object");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create main DirectInput object: OK");

	if (lpdi->CreateDevice(GUID_SysMouse,&lpdidevice, NULL)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create mouse device");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create mouse device: OK");

	if (lpdidevice->SetCooperativeLevel(hwnd,DISCL_EXCLUSIVE | 
										DISCL_FOREGROUND)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Set exclusive cooperative level");
		return(FALSE);
	}
	Log.WriteToLogFile("> Set exclusive cooperative level: OK");

	if (lpdidevice->SetDataFormat(&c_dfDIMouse)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Set mouse data format");
		return(FALSE);
	}
	Log.WriteToLogFile("> Set mouse data format: OK");

	if (lpdidevice->Acquire()!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Aquire mouse");
		return(FALSE);
	}
	Log.WriteToLogFile("> Aquire mouse: OK");

	MouseX=320;
	MouseY=240;

	return(TRUE);
} // end MouseInit

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::KeyboardInit(HWND hwnd)
{
	if (lpdi->CreateDevice(GUID_SysKeyboard,&lpdidevice2, NULL)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Create keyboard device");
		return(FALSE);
	}
	Log.WriteToLogFile("> Create keyboard device: OK");

	if (lpdidevice2->SetCooperativeLevel(hwnd,DISCL_NONEXCLUSIVE|
											  DISCL_BACKGROUND)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Set exclusive cooperative level");
		return(FALSE);
	}
	Log.WriteToLogFile("> Set exclusive cooperative level: OK");

	if (lpdidevice2->SetDataFormat(&c_dfDIKeyboard)!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Set keyboard data format");
		return(FALSE);
	}
	Log.WriteToLogFile("> Set keyboard data format: OK");

	if (lpdidevice2->Acquire()!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Aquire keyboard");
		return(FALSE);
	}
	Log.WriteToLogFile("> Aquire keyboard: OK");

	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GetMouseInfo()
{
	HRESULT hresult;
	
	hresult = lpdidevice->GetDeviceState(sizeof(DIMOUSESTATE), 
										(LPVOID)&mouse_state);
	if (hresult==DIERR_INPUTLOST)
		lpdidevice->Acquire();
	else if (hresult!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Get device state of mouse");
		return(FALSE);
	}

	// move the mouse cursor
	MouseX=(MouseX+=mouse_state.lX);
	MouseY=(MouseY+=mouse_state.lY);
	for(int i=0;i<3;i++)
	{
		if (mouse_state.rgbButtons[i]==128) MouseB[i]=1;
		else MouseB[i]=0;
	}

	// first x boundaries
	if (MouseX >= SCREEN_WIDTH)
		MouseX = SCREEN_WIDTH-1;
	else
	if (MouseX < 0)
		MouseX = 0;

	// now the y boundaries
	if (MouseY >= SCREEN_HEIGHT)
		MouseY= SCREEN_HEIGHT-1;
	else
	if (MouseY < 0)
		MouseY = 0;

	return(TRUE);
} // end GetMouseInfo

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::CheckMouseRange(int x,int y,int x2,int y2,BOOL Button)
{
	
	if(Button)
	{
		if(MouseX>=x && MouseX<=x2 && MouseY>=y && MouseY<=y2 && MouseB[0]) 
			return(TRUE);
	}
	else
	{
		if(MouseX>=x && MouseX<=x2 && MouseY>=y && MouseY<=y2) 
			return(TRUE);
	}

	return(FALSE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::GetKeyboardInfo()
{
	HRESULT hresult;
	
	hresult = lpdidevice2->GetDeviceState(sizeof(keyboard_state), 
										(LPVOID)&keyboard_state);
	if (hresult==DIERR_INPUTLOST)
		lpdidevice2->Acquire();
	else if (hresult!=DI_OK)
	{
		Log.WriteErrorToLogFile("> Error: Get device state of keyboard");
		return(FALSE);
	}

	KeyboardB[0]=0;
	KeyboardB[1]=0;
	KeyboardB[2]=0;
	KeyboardB[3]=0;

	int i2=0;

	if(keyboard_state[DIK_A] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='A';
		i2++;
	}
	if(keyboard_state[DIK_B] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='B';
		i2++;
	}
	if(keyboard_state[DIK_C] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='C';
		i2++;
	}
	if(keyboard_state[DIK_D] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='D';
		i2++;
	}
	if(keyboard_state[DIK_E] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='E';
		i2++;
	}
	if(keyboard_state[DIK_F] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='F';
		i2++;
	}
	if(keyboard_state[DIK_G] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='G';
		i2++;
	}
	if(keyboard_state[DIK_H] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='H';
		i2++;
	}
	if(keyboard_state[DIK_I] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='I';
		i2++;
	}
	if(keyboard_state[DIK_J] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='J';
		i2++;
	}
	if(keyboard_state[DIK_K] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='K';
		i2++;
	}
	if(keyboard_state[DIK_L] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='L';
		i2++;
	}
	if(keyboard_state[DIK_M] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='M';
		i2++;
	}
	if(keyboard_state[DIK_N] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='N';
		i2++;
	}
	if(keyboard_state[DIK_O] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='O';
		i2++;
	}
	if(keyboard_state[DIK_P] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='P';
		i2++;
	}
	if(keyboard_state[DIK_Q] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='Q';
		i2++;
	}
	if(keyboard_state[DIK_R] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='R';
		i2++;
	}
	if(keyboard_state[DIK_S] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='S';
		i2++;
	}
	if(keyboard_state[DIK_T] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='T';
		i2++;
	}
	if(keyboard_state[DIK_U] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='U';
		i2++;
	}
	if(keyboard_state[DIK_V] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='V';
		i2++;
	}
	if(keyboard_state[DIK_W] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='W';
		i2++;
	}
	if(keyboard_state[DIK_X] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='X';
		i2++;
	}
	if(keyboard_state[DIK_Y] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='Z';
		i2++;
	}
	if(keyboard_state[DIK_Z] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='Y';
		i2++;
	}
	if(keyboard_state[DIK_1] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='1';
		i2++;
	}
	if(keyboard_state[DIK_2] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='2';
		i2++;
	}
	if(keyboard_state[DIK_3] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='3';
		i2++;
	}
	if(keyboard_state[DIK_4] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='4';
		i2++;
	}
	if(keyboard_state[DIK_5] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='5';
		i2++;
	}
	if(keyboard_state[DIK_6] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='6';
		i2++;
	}
	if(keyboard_state[DIK_7] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='7';
		i2++;
	}
	if(keyboard_state[DIK_8] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='8';
		i2++;
	}
	if(keyboard_state[DIK_9] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='9';
		i2++;
	}
	if(keyboard_state[DIK_0] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='0';
		i2++;
	}
	if(keyboard_state[DIK_ESCAPE] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=27;
		i2++;
	}		
	if(keyboard_state[DIK_BACK] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=8;
		i2++;
	}		
	if(keyboard_state[DIK_RETURN] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=13;
		i2++;
	}
	if(keyboard_state[DIK_LEFT] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=129;
		i2++;
	}
	if(keyboard_state[DIK_RIGHT] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=130;
		i2++;
	}
	if(keyboard_state[DIK_UP] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=131;
		i2++;
	}
	if(keyboard_state[DIK_DOWN] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=132;
		i2++;
	}
	if((keyboard_state[219] & 0x80) || (keyboard_state[220] & 0x80)) 
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=219;
		i2++;
	}
	if(keyboard_state[56] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=56;
		i2++;
	}
	if(keyboard_state[62] & 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=62;
		i2++;
	}

	if((keyboard_state[27] && 0x80) || (keyboard_state[78] && 0x80))
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='+';
		i2++;
	}

	if(keyboard_state[53] && 0x80 || (keyboard_state[74] && 0x80))
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]='-';
		i2++;
	}
	
	if(keyboard_state[211] && 0x80)
	{
		if(i2>3) return(TRUE);
		KeyboardB[i2]=211;
		i2++;
	}

	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::LoadTextures()
{
	HANDLE SearchHandle;
	WIN32_FIND_DATA finddata;
	int NumFiles=0;
	char name[MAX_PATH];

	SearchHandle = FindFirstFile("textures\\*.bmp",&finddata);	
	do
	{
		sprintf(buffer,"textures\\%s",finddata.cFileName);
		Textures[NumFiles] = new CTexture(buffer);
		NumFiles++;
	}	
	while(FindNextFile(SearchHandle,&finddata)!=FALSE);
	FindClose(SearchHandle);

	NumTextures=NumFiles;

	sprintf(buffer,"> Loaded %d textures",NumFiles);
	Log.WriteToLogFile(buffer);

	Masks[0] = new CTexture("masks\\mask1.msk");
	Masks[1] = new CTexture("masks\\mask2.msk");
	Masks[2] = new CTexture("masks\\mask3.msk");
	
	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::RenderLiquids()
{	
	Icon=Cross;

	for(int i=0;i<128;i++)
	{
		if(Liquids[i])
		{
			if(!CheckBoxes[3]->State)
			{
				if(i!=ActiveLiquid)
					Liquids[i]->RenderOutline(FALSE);
				else Liquids[i]->RenderOutline(TRUE);
			}
		}
	}

	return(TRUE);
}

//////////////////////////////////////////////////////////////////////////////////

BOOL CMain::CheckLiquids()
{
	ActiveLiquid=-1;
	if(CheckBoxes[3]->State) return(TRUE);

	for(int i=0;i<128;i++)
	{
		if(Liquids[i])
		{		
			Liquids[i]->CheckMouse(i);
		}
	}

	return(TRUE);
}

